import logging
from ipaddress import ip_address

import pytest
from pycaracal import Probe, prober, set_log_level


@pytest.mark.asyncio
async def test_probe_async():
    # We observed some segfaults when pycaracal is called from async code
    # and log messages are emitted by the C++ code. Let's ensure that it
    # doesn't happen anymore.
    def gen():
        for _ in range(10_000):
            yield Probe("127.0.0.1", 24000, 33434, 32, "icmp", 0)

    logging.basicConfig(level=logging.DEBUG)
    set_log_level(logging.DEBUG)

    # TODO: Find the proper interface on GitHub actions
    # config = prober.Config()
    # config.set_interface("lo0")
    # config.set_probing_rate(10_000)
    # config.set_sniffer_wait_time(1)
    # prober.probe(config, gen())



import logging
from ipaddress import IPv6Address
from pathlib import Path

from pycaracal import Probe, log_to_stderr, prober, set_log_format, set_log_level


def test_probe():
    p1 = Probe("8.8.4.4", 24000, 33434, 32, "udp", 0)
    p2 = Probe("::ffff:8.8.4.4", 24000, 33434, 32, "udp", 0)
    p3 = Probe("::ffff:8.8.4.4", 24000, 33434, 32, "icmp", 0)
    p4 = Probe(int(IPv6Address("::ffff:8.8.4.4")), 24000, 33434, 32, "icmp", 0)
    assert (
        str(p1)
        == "Probe(dst_addr=8.8.4.4 src_port=24000 dst_port=33434 ttl=32 protocol=udp wait_us=0)"
    )
    assert p1 == p2
    assert p2 != p3
    assert p3 == p4
    assert p1.dst_addr == "8.8.4.4"
    assert p1.src_port == 24000
    assert p1.dst_port == 33434
    assert p1.protocol == "udp"


def test_prober():
    config = prober.Config()
    config.set_output_file_csv("zzz_output.csv")
    config.set_sniffer_wait_time(1)
    probes = [
        Probe("8.8.4.4", 24000, 33434, 32, "icmp", 0),
        Probe("8.8.4.4", 24000, 33434, 32, "udp", 0),
        Probe("8.8.8.8", 24000, 33434, 32, "icmp", 0),
        Probe("8.8.8.8", 24000, 33434, 32, "udp", 0),
    ]
    log_to_stderr()
    set_log_level(logging.DEBUG)
    set_log_format("[%Y-%m-%d %H:%M:%S.%e] [%l] [caracal] %v")
    prober_stats, sniffer_stats, pcap_stats = prober.probe(config, [])
    assert prober_stats.read == 0
    assert pcap_stats.received >= sniffer_stats.received_count
    assert pcap_stats.dropped == 0
    assert pcap_stats.interface_dropped == 0
    prober_stats, sniffer_stats, pcap_stats = prober.probe(config, probes)
    assert prober_stats.read == 4
    assert prober_stats.sent == 4
    assert pcap_stats.received >= sniffer_stats.received_count
    assert pcap_stats.dropped == 0
    assert pcap_stats.interface_dropped == 0
    # This is flaky on GitHub Actions...
    # assert sniffer_stats.received_count >= 1
    input_file = Path("zzz_input.csv")
    input_file.write_text(
        """
8.8.4.4,24000,33434,32,icmp
8.8.4.4,24000,33434,32,udp
8.8.8.8,24000,33434,32,icmp
8.8.8.8,24000,33434,32,udp
    """.strip()
    )
    prober_stats, sniffer_stats, pcap_stats = prober.probe(config, str(input_file))
    assert prober_stats.read == 4
    assert prober_stats.sent == 4
    assert pcap_stats.received >= sniffer_stats.received_count
    assert pcap_stats.dropped == 0
    assert pcap_stats.interface_dropped == 0



from skbuild import setup

setup(
    name="pycaracal",
    version="0.12.0",
    author="Maxime Mouchet",
    author_email="max@maxmouchet.com",
    url="https://github.com/dioptra-io/caracal",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: C++",
        "Programming Language :: Python :: 3",
        "Topic :: Internet",
    ],
    python_requires=">=3.8",
    package_dir={"": "python"},
    packages=["pycaracal"],
    cmake_args=[
        "-DWITH_BINARY=OFF",
        "-DWITH_TESTS=OFF",
    ],
    cmake_install_dir="python/pycaracal",
)



import logging

from pycaracal import Probe, experimental, set_log_level, utilities


def test_prober():
    prober = experimental.Prober(
        utilities.get_default_interface(), 100, 1024 * 1024, 1, True
    )
    probes = [
        Probe("8.8.4.4", 24000, 33434, 32, "icmp", 0),
        Probe("8.8.4.4", 24000, 33434, 32, "udp", 0),
    ]
    set_log_level(logging.DEBUG)
    replies = prober.probe(probes, 1000)
    assert isinstance(replies, list)
